# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; -*-
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
# datreant.data

"""
datreant.data --- numpy and pandas data storage for Treants in HDF5
===================================================================

"""
from datreant.core import Treant, Tree, Bundle, View, Group

from .core import DataFile
from . import pydata, npdata, pddata
from . import tests
from . import limbs
from . import agglimbs

__all__ = ['Treant', 'Group', 'Tree', 'Leaf', 'Bundle']
__version__ = "0.6.0"  # NOTE: keep in sync with RELEASE in setup.py
